#include "WatchFace.h"
#include "SevenSegment.h"
#include <cmath>

RTC_DATA_ATTR bool WatchFace::m_menuSetup = false;
RTC_DATA_ATTR bool WatchFace::m_inMenu = false;
Menu WatchFace::m_menu;

static const std::vector<MenuPage> menuPages = {
	{
		"Watchy",
		ALIGNMENT_CENTER,
		"",
		ALIGNMENT_CENTER,
		{
			{
				"Sync NTP",
				nullptr,
				0
			}
		}
	}
};

void WatchFace::Setup() // Called after hardware is set up
{
	if (!m_menuSetup) {
		m_menuSetup = true;
		m_menu.Init(m_display);
	}

	m_menu.SetPages(menuPages);
	m_menu.SetExitCallback(std::bind(&WatchFace::MenuExited, this));
}

void WatchFace::HandleButtonPress(uint64_t buttonMask)
{
	if (m_inMenu) {
		m_menu.HandleButtonPress(buttonMask);
	} else {
		if (buttonMask & MENU_BTN_MASK) {
			m_inMenu = true;
			m_menu.Redraw(false);
		}
	}
}

void WatchFace::DrawWatchFace(bool partialRefresh)
{
	if (m_inMenu) {
		return;
	}

	m_display.setFullWindow();

	if (!partialRefresh) {
		m_display.clearScreen();
	}

	m_display.fillScreen(GxEPD_WHITE);
	DrawBatteryIcon();

	tmElements_t currentTime;
	m_RTC.read(currentTime, TZ_OFFSET);
	SevenSegment sevenSegment(30, 60, 6, 5, 5);

	if (currentTime.Hour < 10) {
		sevenSegment.DrawDigit(m_display, 0, 10, 75, GxEPD_BLACK);
	} else {
		sevenSegment.DrawDigit(m_display, currentTime.Hour / 10, 10, 75, GxEPD_BLACK);
	}

	sevenSegment.DrawDigit(m_display, currentTime.Hour % 10, 50, 75, GxEPD_BLACK);

	if (currentTime.Minute < 10) {
		sevenSegment.DrawDigit(m_display, 0, 100, 75, GxEPD_BLACK);
	} else {
		sevenSegment.DrawDigit(m_display, currentTime.Minute / 10, 100, 75, GxEPD_BLACK);
	}

	sevenSegment.DrawDigit(m_display, currentTime.Minute % 10, 140, 75, GxEPD_BLACK);

	m_display.fillRect(87, 90, 5, 5, GxEPD_BLACK);
	m_display.fillRect(87, 110, 5, 5, GxEPD_BLACK);

	m_display.display(partialRefresh);
}

void WatchFace::DrawBatteryIcon()
{
	m_display.fillRect(200 - 48, 5, 43, 23, GxEPD_BLACK);
	m_display.fillRect(200 - 46, 7, 39, 19, GxEPD_WHITE);
	float VBAT = GetBatteryVoltage();
	float level = (VBAT - 3.6f) / 0.6f;
	if (level > 1.0f) {
		level = 1.0f;
	} else if (level < 0.0f) {
		level = 0.0f;
	}
	level = 0.5f - sin(asin(1.0f - 2.0f * level) / 3.0f);

	m_display.fillRect(200 - 44, 9, (int)std::round(35.0f * level), 15, GxEPD_BLACK);
}

void WatchFace::MenuExited()
{
	if (m_inMenu) {
		m_inMenu = false;
		DrawWatchFace(false);
	}
}